import matplotlib.pyplot as plt
import numpy as np
import fastcomtec_api

def printkw(**keywordarguments) -> None:
    for kw in keywordarguments:
        print(kw, f"{keywordarguments[kw]:e}")

def print_kw_enum(**keywordarguments) -> None:
    for kw in keywordarguments:
        print(kw)
        for i, el in enumerate(keywordarguments[kw]): 
            print(f"{i:>2}:", el)
        print("----------------------------")

def tt_calculations():
    bin_widths = 2**np.arange(0, 25)
    fundamental_time_resolution = 400*10**(-12)
    time_resolutions = fundamental_time_resolution * bin_widths
    bins_per_second = 1 / time_resolutions

    print_kw_enum(bin_widths = bin_widths, time_res = time_resolutions, bins_per_s = bins_per_second)

    bin_selection = 24
    total_measurement_duration_sec = 300

    printkw(bins = bins_per_second[bin_selection] * total_measurement_duration_sec)

    # print_kw(total_time = 29824*time_resolutions[22])

def calculate_dark_count_histogram(file: str, name: str, total_time_sec: int, num_measurements: int):
    
    data = np.loadtxt(file, dtype=np.int32, skiprows=1)
    # print(data.shape)
    data_50s_split = np.array_split(data, num_measurements)
    counts_per_measurement = []
    for ar in data_50s_split:
        # print(ar.shape)
        counts_per_measurement.append(ar.sum())
    bins=np.arange(np.array(counts_per_measurement).max() + 1) 
    plt.hist(counts_per_measurement, bins, density=True, edgecolor='black', linewidth=1.2)
    plt.xticks(bins+0.5, [str(b) for b in bins])
    plt.title(f"SPCM {name} Histogram\n{num_measurements} $\cdot$ {total_time_sec/num_measurements}s measurements")
    plt.ylabel("P(n)")
    plt.xlabel("n")
    plt.show()

def calculate_test_diode_histogram(file: str, name: str, total_time_sec: int, num_measurements: int):
    print("Loading data ...")
    data = np.loadtxt(file, dtype=np.int32, skiprows=1)
    # print(data.shape)
    data_50s_split = np.array_split(data, num_measurements)
    counts_per_measurement = []
    print("Evaluating counts ...")
    for ar in data_50s_split:
        # print(ar.shape)
        counts_per_measurement.append(ar.sum())
    min_count = np.array(counts_per_measurement).min()
    max_count = np.array(counts_per_measurement).max() + 1
    # bins=np.arange(max_count) 
    bins = np.linspace(min_count, max_count + 1, num=25)
    bin_width = (max_count - min_count)/25
    print("Plotting histogram ...")
    plt.hist(counts_per_measurement, bins, edgecolor='black', linewidth=1.2)
    plt.xticks(bins+bin_width * 0.5, [f"{int(b):.5e}" for b in bins], rotation=90)
    plt.title(f"SPCM {name} Histogram\n{num_measurements} $\cdot$ {total_time_sec/num_measurements}s measurements")
    plt.ylabel("P(n)")
    plt.xlabel("n")
    print("Done")
    plt.show()
    print("Exiting")

def calculate_darkcount_histogram_tripple():
    data_100_0 = np.loadtxt(r"C:\Users\admin\benjamin_code\FastComTec_self\measurements\100s_darkcount_data_TDAT1.mpa", dtype=np.int32, skiprows=1)
    print(data_100_0.shape)
    data_100_1 = np.loadtxt(r"C:\Users\admin\benjamin_code\FastComTec_self\measurements\100s-1_darkcount_data_TDAT1.mpa", dtype=np.int32, skiprows=1)
    print(data_100_1.shape)
    data_100_2 = np.loadtxt(r"C:\Users\admin\benjamin_code\FastComTec_self\measurements\100s-2_darkcount_data_TDAT1.mpa", dtype=np.int32, skiprows=1)
    print(data_100_2.shape)
    data_combined = np.hstack((data_100_0, data_100_1, data_100_2))
    seconds = 300
    data_split = np.array_split(data_combined, seconds)

    counts_per_sec = []
    for ar in data_split:
        # print(ar.shape)
        counts_per_sec.append(ar.sum())
    # print(len(data_split))
    bins=np.arange(25) 
    plt.hist(counts_per_sec, bins, density=True)
    plt.xticks(bins, [str(b) for b in bins])
    plt.title(f"SPCM Dark Count Histogram ({seconds}x1s measurements)")
    plt.ylabel("P(n)")
    plt.xlabel("n")
    plt.tight_layout()
    plt.show()

def light_calculations():
    P0 = 2.5 * 10** (-3)
    wavelen = 780 * 10**(-9)
    h = 6 * 10**(-34)
    c = 3 * 10**8
    photon_energy = c/wavelen * h
    num_photons = P0 / photon_energy
    attenuation_dB = -5
    printkw(photon_energy = photon_energy)
    printkw(num_photons = num_photons)
    printkw(attenuated = num_photons * 10**attenuation_dB)


plt.rcParams.update({'figure.autolayout': True})

### Split file
# fastcomtec_api.separate_config_and_data_per_channel(r"C:\Users\admin\benjamin_code\FastComTec_self\measurements\Thorlabs Laser Diode\50s-2_5OD.mpa")

### Dark counts
# calculate_dark_count_histogram(r"C:\Users\admin\benjamin_code\FastComTec_self\measurements\Dark counts\50s_darkcount_data_TDAT1.mpa" , "Dark Count", 50, 50)
# calculate_dark_count_histogram(r"C:\Users\admin\benjamin_code\FastComTec_self\measurements\Dark counts\100s_darkcount_data_TDAT1.mpa", "Dark Count", 100, 100)
# calculate_darkcount_histogram_tripple()

### Thorlabs Diode
light_calculations()
# calculate_test_diode_histogram(r"C:\Users\admin\benjamin_code\FastComTec_self\measurements\Thorlabs Laser Diode\50s_5OD_data_TDAT1.mpa", "780nm Diode Counts", 50, 50)



### Calculations
# tt_calculations()
# light_calculations()

